package main;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;

import util.Array;

import lemonade.ScaledLemonadeUtility;

/**
 * A main class for calculating utilities.
 * Answers queries interactively.
 * @author maz
 *
 */
public class UtilityCalculator {

  /**
   * Start a utility calculator.
   * The input format is:
   * <ACTION> <ACTION> <ACTION>
   * where each <ACTION> is a number between 0 and 11.
   * 
   * @param args
   * @throws IOException 
   */
  public static void main(String[] args) throws IOException {
    Reader r = new InputStreamReader(System.in);
    BufferedReader in = new BufferedReader(r);
    String line;
    ScaledLemonadeUtility util = new ScaledLemonadeUtility(12);
    while((line=in.readLine())!=null){
      try{
        String[] inputStrings = line.split("\\s+");
        if (inputStrings.length!=3){
          throw new NumberFormatException("Too many arguments.");
        }
        int[] inputInts = new int[inputStrings.length];
        for(int i=0;i<inputInts.length;i++){
          inputInts[i] = Integer.parseInt(inputStrings[i]);
          if (inputInts[i]>=util.getIslandSize()){
            throw new NumberFormatException("Integer too big.");
          }
          if (inputInts[i]<0){
            throw new NumberFormatException("Integer too small.");
          }

        }
        int[] output = util.getUtility(inputInts); 
        System.out.println(Array.arrayToNakedString(output));
      } catch (NumberFormatException nfe){
        System.err.println(nfe.getMessage());
        System.err.println("Format error: input should be:");
        System.err.println("<ACTION> <ACTION> <ACTION>");
        System.err.println("where each action is a number between 0 and "+(util.getIslandSize()-1));
      }
    }
  }

}
