package net;


import java.io.IOException;
import java.io.OutputStream;
import java.net.Socket;
import java.net.SocketException;

import util.Array;

public class Agent implements Runnable{
  Socket s;
  InputStreamListener is;
  OutputStream os;
  int actionObserved;
  GameServer g;
  boolean gameOver;
  
  
  public String toString(){
    if (s!=null){
      return ""+s.getInetAddress()+":"+s.getPort();
    } else {
      return "?";
    }
  }
  public Agent(GameServer g, Socket s) {
    this.g = g;
    this.s = s;
    try{
    is = new InputStreamListener(s.getInputStream(),HumanRemoteClient.messageTerminator);
    is.addChatListener(g);
    os = s.getOutputStream();
    } catch (IOException io){
      is = null;
      os = null;
    }
    actionObserved = -1;
    gameOver = false;
    Thread t = new Thread(this);
    t.start();
  }
  
  public synchronized void setAction(int action){
    if (actionObserved==-1){
      actionObserved=action;
    }
  }
  
  public synchronized int getActionObserved(){
    int result = actionObserved;
    actionObserved = -1;
    return result;
  }
  
  public synchronized boolean lookingForAction(){
    return (actionObserved==-1);
  }
    
  public void observeOutcome(int[] actions){
    try{
    String outcome = Array.arrayToNakedString(actions);
    if (g.lastRound){
      sendMessage("GAMEOVER:"+outcome);
    } else {
      sendMessage("OUTCOME:"+outcome);
    }
    } catch (IOException io){
      gameOver = true;
      System.err.println("Issue with the outcome");
      io.printStackTrace();
    }
  }
  
  
  
  @Override
  public void run() {
    try {
      System.out.println("Agent starts run...");
      while(!g.handshakeComplete){
        Thread.sleep(100);
      }
      while(!gameOver){
      String messageReceived = receiveMessage();
      setAction(Integer.parseInt(messageReceived));
      while(!lookingForAction() && !gameOver){
        Thread.sleep(100);
      }
      }
    }  catch (Exception e) {
      System.err.println("Exception: error communicating with agent");
      e.printStackTrace();
      gameOver = true;
    } 
  }
  
  /**
   * Send a message to the server. Appends a message terminator (CR LF).
   * If the match is over, send nothing.
   */
  public synchronized void sendMessage(String message) throws SocketException,IOException{
    assert(message!=null);
    showVerbose("SERVER SENDS:"+message);
    message = message + HumanRemoteClient.messageTerminator;
    byte[] messageData = message.getBytes();
    if (!gameOver && os!=null){
        os.write(messageData);
    }
    if (os==null){
      throw new IOException("Not connected");
    }
  }
  
  
  /**
   * Receive a message from the server. Removes a message terminator (CR LF).
   * @throws InterruptedException 
   */
  public String receiveMessage() throws SocketException, IOException, InterruptedException{
    if (is==null){
      throw new IOException("Not connected");
    }
    return is.receiveMessage();  
  }
  
  public String peekAtMessage() throws IOException, InterruptedException{
    if (is==null){
      throw new IOException("InputStreamListener not initialized");
    }
    return is.peekAtMessage();
  }
  
  public void showVerbose(String string){
    System.err.println(string);
  }
  /**
   * Test if the message is complete (contains a terminal character)
   */
  public boolean isComplete(String result){
      return result.endsWith(HumanRemoteClient.messageTerminator);
  }

  
  

}
